@@ -8,6 +8,7 @@ use crate::borsh::BorshSerialize;
 use crate::logging::pretty_hash;
 use crate::serialize::{from_base, to_base, BaseDecode};
 
+#[cfg_attr(feature = "deepsize_feature", derive(deepsize::DeepSizeOf))]
 #[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, derive_more::AsRef, derive_more::AsMut)]
 #[as_ref(forward)]
 #[as_mut(forward)]
@@ -67,7 +68,7 @@ impl<'de> Deserialize<'de> for CryptoHash {
             return Err(serde::de::Error::custom("incorrect length for hash"));
         }
         from_base(&s)
-            .and_then(CryptoHash::try_from)
+            .and_then(|f| CryptoHash::try_from(f.as_slice()))
             .map_err(|err| serde::de::Error::custom(err.to_string()))
     }
 }
@@ -77,7 +78,7 @@ impl std::str::FromStr for CryptoHash {
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         let bytes = from_base(s).map_err::<Self::Err, _>(|e| e.to_string().into())?;
-        Self::try_from(bytes)
+        Self::try_from(bytes.as_slice())
     }
 }
 
@@ -85,20 +86,7 @@ impl TryFrom<&[u8]> for CryptoHash {
     type Error = Box<dyn std::error::Error>;
 
     fn try_from(bytes: &[u8]) -> Result<Self, Self::Error> {
-        if bytes.len() != 32 {
-            return Err("incorrect length for hash".into());
-        }
-        let mut buf = [0; 32];
-        buf.copy_from_slice(bytes);
-        Ok(CryptoHash(buf))
-    }
-}
-
-impl TryFrom<Vec<u8>> for CryptoHash {
-    type Error = Box<dyn std::error::Error>;
-
-    fn try_from(v: Vec<u8>) -> Result<Self, Self::Error> {
-        <Self as TryFrom<&[u8]>>::try_from(v.as_ref())
+        Ok(CryptoHash(bytes.try_into()?))
     }
 }
 
@@ -164,14 +152,14 @@ mod tests {
     #[test]
     fn test_serialize_success() {
         let hash = hash(&[0, 1, 2]);
-        let s = Struct { hash: hash.into() };
+        let s = Struct { hash };
         let encoded = serde_json::to_string(&s).unwrap();
         assert_eq!(encoded, "{\"hash\":\"CjNSmWXTWhC3EhRVtqLhRmWMTkRbU96wUACqxMtV1uGf\"}");
     }
 
     #[test]
     fn test_serialize_default() {
-        let s = Struct { hash: CryptoHash::default().into() };
+        let s = Struct { hash: CryptoHash::default() };
         let encoded = serde_json::to_string(&s).unwrap();
         assert_eq!(encoded, "{\"hash\":\"11111111111111111111111111111111\"}");
     }
@@ -179,21 +167,21 @@ mod tests {
     #[test]
     fn test_deserialize_default() {
         let encoded = "{\"hash\":\"11111111111111111111111111111111\"}";
-        let decoded: Struct = serde_json::from_str(&encoded).unwrap();
-        assert_eq!(decoded.hash, CryptoHash::default().into());
+        let decoded: Struct = serde_json::from_str(encoded).unwrap();
+        assert_eq!(decoded.hash, CryptoHash::default());
     }
 
     #[test]
     fn test_deserialize_success() {
         let encoded = "{\"hash\":\"CjNSmWXTWhC3EhRVtqLhRmWMTkRbU96wUACqxMtV1uGf\"}";
-        let decoded: Struct = serde_json::from_str(&encoded).unwrap();
-        assert_eq!(decoded.hash, hash(&[0, 1, 2]).into());
+        let decoded: Struct = serde_json::from_str(encoded).unwrap();
+        assert_eq!(decoded.hash, hash(&[0, 1, 2]));
     }
 
     #[test]
     fn test_deserialize_not_base58() {
         let encoded = "\"---\"";
-        match serde_json::from_str(&encoded) {
+        match serde_json::from_str(encoded) {
             Ok(CryptoHash(_)) => assert!(false, "should have failed"),
             Err(_) => (),
         }
@@ -208,8 +196,8 @@ mod tests {
             format!("\"{}\"", "1".repeat(33)),
             format!("\"{}\"", "1".repeat(1000)),
         ] {
-            match serde_json::from_str::<CryptoHash>(&encoded) {
-                Err(e) if e.to_string() == "incorrect length for hash" => {}
+            match serde_json::from_str::<CryptoHash>(encoded) {
+                Err(e) => if e.to_string() == "could not convert slice to array" {},
                 res => assert!(false, "should have failed with incorrect length error: {:?}", res),
             };
         }
